package rabbit.cache;

// $Id: FiledHook.java,v 1.2 2005/05/05 15:29:07 robo Exp $

import java.io.*;
import java.lang.ref.*;
import java.util.zip.*;

/** A class to store the cache entrys data hook on file. 
 *  A HTTPHeader is a big thing so it is nice to write it to disk. 
 */
class FiledHook implements Serializable {
    private static final long serialVersionUID = 20050430;

    public FiledHook () {
    }

    protected String getExtension () {
	return ".hook";
    }

    /** Get the hooked data. 
     */
    public Object getData (NCache cache, NCacheEntry entry) {
	return readHook (cache, entry);
    }

    /** Read the hooked data. 
     */
    protected Object readHook (NCache cache, NCacheEntry entry) {
	String name = cache.getEntryName (entry) + getExtension ();
	return readHook (name);
    }

    /** Read the hooked data. 
     */
    protected Object readHook (String name) {
	ObjectInputStream is = null;
	try {
	    File f = new File (name);
	    if (!f.exists())
		return null;
	    is = new ObjectInputStream (new GZIPInputStream (new FileInputStream (f)));
	    Object hook = is.readObject ();
	    return hook;
	} catch (ClassNotFoundException e) {
	    e.printStackTrace ();
	} catch (IOException e) {
	    e.printStackTrace ();
	} finally {
	    if (is != null) {
		try {
		    is.close ();
		} catch (IOException e) {
		    e.printStackTrace ();
		}
	    }
	}
	return null;
    }

    /** Set the hooked data. 
     */
    protected void storeHook (NCache cache, NCacheEntry entry, Object hook) {
	ObjectOutputStream os = null;
	try {
	    String name = cache.getEntryName (entry) + getExtension ();
	    os = new ObjectOutputStream (new GZIPOutputStream (new FileOutputStream (name)));
	    os.writeObject (hook);
	    os.flush ();
	} catch (IOException e) {
	    e.printStackTrace ();
	} finally {
	    if (os != null) {
		try {
		    os.close ();
		} catch (IOException e) {
		    e.printStackTrace ();
		}
	    }
	}
    }
}
