package rabbit.cache;

// $Id: FiledKey.java,v 1.2 2005/05/05 15:29:07 robo Exp $

import java.io.*;

/** A class that stores cache keys in compressed form. 
 */
class FiledKey extends FiledHook {
    private static final long serialVersionUID = 20050430;

    protected int hashCode; // the hashCode for the contained object.
    private long id;
    protected transient NCache cache;    
    
    protected String getExtension () {
	return ".key";
    }

    protected void setCache (NCache cache) {
	this.cache = cache;
    }

    protected void storeKey (NCache cache, NCacheEntry entry, Object key) {
	setCache (cache);
	hashCode = key.hashCode ();
	id = entry.getId ();
	storeHook (cache, entry, key);
    }

    private String getFileName () {
	return cache.getEntryName (id, true) + getExtension (); 
    }
    
    /** Get the hashCode for the contained key object. */
    public int hashCode () {
	return hashCode;
    }

    /** Check if the given object is equal to the contained key. */
    public boolean equals (Object data) {
	Object myData = getData ();
	if (data != null && data instanceof FiledKey) {
	    data = ((FiledKey)data).getData ();
	}
	if (myData != null) {
	    return myData.equals (data);
	} else {
	    return data == null;
	}
    }
    
    /** Get the actual key object. */
    public Object getData () {
	return readHook (getFileName ());
    }

    /** Get the unique id for this object. */
    public long getId () {
	return id;
    }

    public String toString () {
	return "FiledKey: " + hashCode + ", " + getFileName ();
    }
}
