package rabbit.filter;

// $Id: BlinkFilter.java,v 1.10 2005/08/02 20:50:21 robo Exp $

import java.util.List;
import rabbit.html.*;
import rabbit.http.HTTPHeader;
import rabbit.proxy.Connection;

/** A filter that removes the blink and /blink tags.
 */
public class BlinkFilter extends HTMLFilter {

    // For the factory.
    public BlinkFilter () {
    }

    /** Create a new BlinkFilter for the given request, response pair.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response being sent.
     */
    public BlinkFilter (Connection con, HTTPHeader request, HTTPHeader response) {	
	super (con, request, response);
    }

    @Override public HTMLFilter newFilter (Connection con, 
					   HTTPHeader request, 
					   HTTPHeader response) {
	return new BlinkFilter (con, request, response);
    }

    /** Remove blink tags.
     * @param block the part of the html page we are filtering.
     */
    public void filterHTML (HTMLBlock block) {
	List<Token> tokens = block.getTokens ();
	int tsize = tokens.size ();
	for (int i = 0; i < tsize; i++) {
	    Token t = tokens.get (i);
	    switch (t.getType ()) {
	    case Token.TAG:
		Tag tag = t.getTag ();
		TagType tt = tag.getTagType ();
		if (tt == TagType.BLINK || 
		    tt == TagType.SBLINK) {
		    block.removeToken (i);		    
		}
		break;
	    }
	}
    } 
}
