package rabbit.filter;

// $Id: SimpleTagFilter.java,v 1.4 2005/08/02 20:50:21 robo Exp $

import java.util.List;
import rabbit.html.HTMLBlock;
import rabbit.html.Token;
import rabbit.html.Tag;
import rabbit.http.HTTPHeader;
import rabbit.proxy.Connection;

/** A class that inserts some text and links at the top of a page.
 *  Useful for inserting links to unfiltered page.
 */
public abstract class SimpleTagFilter extends HTMLFilter {
    
    // For the factory.
    public SimpleTagFilter () {
    }

    /** Create a new SimpleTagFilter for the given request, response pair.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response being sent.
     */
    public SimpleTagFilter (Connection con, HTTPHeader request, HTTPHeader response) {	
	super (con, request, response);
    }

    /** Iterate over all tags and call handleTag on them. 
     * @param block the part of the html page we are filtering.
     */
    public void filterHTML (HTMLBlock block) {
	List<Token> tokens = block.getTokens ();
	int tsize = tokens.size ();
	for (int i = 0; i < tsize; i++) {
	    Token t = tokens.get (i);
	    switch (t.getType ()) {
	    case Token.TAG:
		Tag tag = t.getTag ();
		handleTag (tag, block, i);
	    }
	}
    }

    /** Handle a tag. 
     * @param tag the Tag to handle.
     * @param block the current HTMLBlock
     * @param tokenIndex the index of the current Token
     */
    public abstract void handleTag (Tag tag, HTMLBlock block, int tokenIndex);
}
