package rabbit.handler;

// $Id: GZIPHandler.java,v 1.11 2005/08/02 20:50:21 robo Exp $

import java.io.IOException;
import java.util.zip.GZIPOutputStream;
import rabbit.http.HTTPHeader;
import rabbit.io.HTTPInputStream;
import rabbit.io.MultiOutputStream;
import rabbit.proxy.Connection;
import rabbit.util.Logger;
import rabbit.util.SProperties;

/** This class is used to gzip streams.
 */
public class GZIPHandler extends BaseHandler{
    private boolean compress = true;
    private GZIPOutputStream gz = null;
    private MultiOutputStream origClientstream = null;
    private boolean isCompressing = false;

    // For creating the factory.
    public GZIPHandler () {	
    }

    /** Create a new GZIPHansler for the given request.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response.
     * @param contentstream the stream to read data from.
     * @param clientstream the stream to write data to.
     * @param maycache May we cache this request? 
     * @param mayfilter May we filter this request?
     * @param size the size of the data beeing handled.
     */
    public GZIPHandler (Connection con, 
			HTTPHeader request, 
			HTTPHeader response, 
			HTTPInputStream contentstream, 
			MultiOutputStream clientstream,
			boolean maycache,
			boolean mayfilter,
			long size) {
	super (con, request, response, contentstream, clientstream, maycache, mayfilter, size);
	
	if (compress) {
	    String gzip = response.getHeader ("Content-Encoding");
	    isCompressing = !(gzip != null 
			      && (gzip.equalsIgnoreCase ("gzip") 
				  || gzip.equalsIgnoreCase ("compress")
				  || gzip.equalsIgnoreCase ("deflate")));
	    if (isCompressing) {
		response.removeHeader ("Content-Length");
		response.setHeader ("Content-Encoding", "gzip");
		if (!con.getChunking ())
		    con.setKeepalive (false);
	    } else {
		this.mayfilter = false;
	    }
	}
    }

    @Override public Handler getNewInstance (Connection connection,
					     HTTPHeader header, 
					     HTTPHeader webheader, 
					     HTTPInputStream contentStream, 
					     MultiOutputStream out, 
					     boolean maycache, 
					     boolean mayfilter, 
					     long size) {
	GZIPHandler gz = new GZIPHandler (connection, header, webheader, 
					  contentStream, out, maycache, 
					  mayfilter, size);
	gz.compress = compress;
	return gz;
    }

    protected boolean clientStreamHasCache () {
	return origClientstream != null && 
	    origClientstream.containsStream (cacheStream);
    }

    /** This method is used to prepare the stream for the data being sent.
     *  This method sets the stream up for gzipping.
     */
    protected void prepareStream () throws IOException {
	if (isCompressing) {
	    origClientstream = clientstream;
	    gz = new GZIPOutputStream (clientstream);
	    clientstream = new MultiOutputStream (gz);
	}
    }
    
    /** This method is used to finish the stream for the data being sent.
     *  This method finishes the gzipping.
     */
    protected void finishStream () throws IOException {
	if (isCompressing)
	    gz.finish ();
    }

    /** Setup this class.
     * @param prop the properties of this class.
     */
    @Override public void setup (Logger logger, SProperties prop) {
	String comp = prop.getProperty ("compress", "true");
	if (comp.equalsIgnoreCase ("false"))
	    compress = false;
	else
	    compress = true;
    }
}
