package rabbit.meta;

// $Id: Connections.java,v 1.9 2005/08/01 12:43:45 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.html.*;
import rabbit.http.*;
import rabbit.io.*;
import rabbit.proxy.*;
import rabbit.proxy.Proxy;

/** The Metapage that shows the external connections the proxy 
 *  has keepalive to.
 */
public class Connections implements MetaHandler {

    /** default constructor */
    public Connections () {
    }

    /** handler of a MetaPage, that is a special page the proxy supports (like the status page).
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param header the http request header
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param con the Connection that is serving the request.
     */
    public void handle (InputStream input, 
			MultiOutputStream output, 
			HTTPHeader header, 
			Properties htab, 
			Connection con) {
	try {      
	    ConnectionHandler ch = con.getProxy ().getConnectionHandler ();
	    output.writeHTTPHeader (con.getResponseHandler ().getHeader ());
	    StringBuilder res = 
		new StringBuilder (HTMLPage.getPageHeader (con, "Current external connections") +
				  "Cleaner is running?: " + ch.isCleanerRunning () + "<br>\n" +
				  "Keepalive is set to: " + (ch.getKeepaliveTime () / 1000) + " s.<br><br><br>\n" +
				  HTMLPage.getTableHeader (100, 1) +
				  HTMLPage.getTableTopicRow () + 
				  "<P><H1>keepalive connections</H1></P>\n" + 
				  "<th width=\"30%\">InetAddress</th><th width=\"20%\">Port</th>" +
				  "<th width=\"50%\">#Connection</th>\n");
	    
	    Iterator e = ch.getAddresses ();
	    while (e.hasNext ()) {
		ConnectionHandler.Address a = (ConnectionHandler.Address)e.next ();
		List pool = ch.getPool (a);
		res.append ("<tr><td>" + a.ia + 
			    "</td><td>" + a.port + 
			    "</td><td>" + pool.size () + "</td></tr>\n");
	    }
	    res.append ("</table><br>\n");
	    res.append ("<P><H1>Pipelined connections</H1></P>\n");
	    res.append (HTMLPage.getTableHeader (100, 1) +
			HTMLPage.getTableTopicRow () + 
			"<th width=\"30%\">InetAddress</th><th width=\"20%\">Port</th>" +
			"<th width=\"50%\">#Connection</th>\n");
	    e = ch.getPipelineAddresses ();
	    while (e.hasNext ()) {
		ConnectionHandler.Address a = (ConnectionHandler.Address)e.next ();
		List pool = ch.getPipelinePool (a);
		res.append ("<tr><td>" + a.ia + 
			    "</td><td>" + a.port + 
			    "</td><td>" + pool.size () + "</td></tr>\n");
	    }	    

	    res.append ("</table>\n</body></html>\n");
	    output.write (res.toString ().getBytes ());
	} catch (IOException e) {
	    con.getProxy ().logError ("Couldnt write status");
	}
    }
}
