package rabbit.meta;

// $Id: SimpleHandler.java,v 1.2 2005/08/01 12:43:45 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.filter.*;
import rabbit.html.*;
import rabbit.http.*;
import rabbit.io.*;
import rabbit.proxy.*;
import rabbit.util.*;

/** A base class for simple meta pages.
 */
public abstract class SimpleHandler implements MetaHandler {
    /** default constructor */
    public SimpleHandler () {
    }
    
    /** handler of a MetaPage, that is a special page the proxy supports (like the status page).
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param header the http request header
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param con the Connection that is serving the request.
     */
    public void handle (InputStream input, 
			MultiOutputStream output, 
			HTTPHeader header, 
			Properties htab, 
			Connection con) {

	String content = header.getContent ();
	if (content == null) {
	    byte[] contentArr = header.getContentArr ();
	    if (contentArr != null)
		content = new String (contentArr);
	}
	if (content != null) {
	    String mark = getMark ();
	    int i = content.indexOf (mark);
	    if (i > -1) {
		int j = content.indexOf ('&', i);
		if (j == -1)
		    j = content.length ();
		if (j > -1) {
		    String userstring = content.substring (i + mark.length (),j);
		    userstring = Coder.URLdecode (userstring);
		    handleData (con, userstring);
		}
	    }
	}
	sendPage (output, con);
    }

    /** Get the mark.
     */
    public abstract String getMark ();

    /** Handle the user data. 
     * @param con the Connection doing the transaction.
     * @param data the url decoded data.
     */
    public abstract void handleData (Connection con, String data);

    /** Send the response page.
     * @param output the stream to send the page on
     * @param con the Connection handling the request
     */
    public abstract void sendPage (MultiOutputStream output, Connection con);
}
