package rabbit.proxy;

// $Id: PartialCacher.java,v 1.4 2005/08/03 17:00:54 robo Exp $

import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.StringTokenizer;
import rabbit.http.HTTPHeader;
import rabbit.util.Logger;

/** This class can save partial data to the correct places in the files.
 */
public class PartialCacher extends OutputStream {
    private RandomAccessFile file; 
    private HTTPHeader response;
    private int start;
    private int end;
    private int total;

    public PartialCacher (Logger logger, String filename, HTTPHeader response) 
	throws IOException {
	file = new RandomAccessFile (filename, "rw");
	this.response = response;
	
	//Content-Range: 0-4/25\r\n
	String cr = response.getHeader ("Content-Range");
	if (cr != null) {
	    if (cr.startsWith ("bytes "))
		cr = cr.substring (6);
	    StringTokenizer st = new StringTokenizer (cr, "-/");
	    if (st.countTokens () == 3) {
		try {
		    start = Integer.parseInt (st.nextToken ());
		    end = Integer.parseInt (st.nextToken ());
		    total = Integer.parseInt (st.nextToken ());
		    file.seek (start);
		} catch (NumberFormatException e) {
		    logger.logError (Logger.WARN, "bad content range: " + e);
		}
	    }
	}
    }
    
    public void close () throws IOException {
	file.close ();
    }

    public void flush () throws IOException {	
    }

    public void write (int b) throws IOException {
	byte[] buf = new byte[1];
	buf[0] = (byte)b;
	write (buf);
    }

    public void write (byte[] b) throws IOException {
	write (b, 0, b.length);
    }

    public void write (byte[] b, int off, int len) throws IOException {
	file.write (b, off, len);
    }

    public int getStart () {
	return start;
    }
    
    public int getEnd () {
	return end;
    }

    public int getTotal () {
	return total;
    }
}
