package rabbit.spider;

// $Id: WebSpider.java,v 1.4 2005/08/01 12:43:45 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;

import rabbit.io.*;
import rabbit.html.*;
import rabbit.http.*;
import rabbit.proxy.*;
import rabbit.proxy.Proxy;

import org.xbill.DNS.*;

/** A web spider that can be used to test rabbit. 
 */
public class WebSpider {

    private List<Worker> workers;
    private int sleeptime;
    private String proxyHost;
    private int proxyPort;
    private static Proxy proxy;

    private LinkedList<URL> urlsToGet = new LinkedList<URL> ();
    private Map<URL, Integer> failures = new WeakHashMap<URL, Integer> ();

    private static final String spiderId = 
    "RabbIT web spider, http://www.khelekore.org/rabbit/web-spider.shtml";
    private Map<String, URL> history = new HashMap<String, URL> ();
    
    public static void main (String[] args) {
	if (args.length < 5) {
	    System.err.println ("usage: WebSpider <threads> <sleeptime seconds> proxyhost proxyport url [url*]");
	    System.exit (-1);
	}
	int threads = Integer.parseInt (args[0]);
	int sleeptime = Integer.parseInt (args[1]) * 1000;
	String sproxy = args[2];
	int proxyPort = Integer.parseInt (args[3]);
	
	/* ugh, this is kinda ugly, but needed for the connection handler. */
	try {
	    proxy = new Proxy ();
	    proxy.setProxy (sproxy);
	    proxy.setProxyPort (proxyPort);
	} catch (UnknownHostException e) {
	    System.err.println ("unable to find proxy host: '" + sproxy + "'");
	    e.printStackTrace ();
	    System.exit (-1);
	}
	
	List<URL> urls = new ArrayList<URL> (args.length - 4);
	for (int i = 4; i < args.length; i++) {
	    try {
		urls.add (new URL (args[i]));
	    } catch (MalformedURLException e) {
		System.err.println ("bad url: " + args[i] + ": " + e);
	    }
	}
	if (urls.size () > 0) {
	    WebSpider ws = new WebSpider (threads, sleeptime, sproxy, proxyPort, urls);
	} else {
	    System.err.println ("unable to parse urls => nothing to do"); 
	    System.exit (-1);
	}
    }
    
    protected WebSpider (int threads, int sleeptime, String proxyHost, int proxyPort, List startURLS) {
	this.sleeptime = sleeptime;
	this.proxyHost = proxyHost;
	this.proxyPort = proxyPort;
	workers = new ArrayList<Worker> (threads);
	for (int i = 0; i < threads; i++) {
	    Worker worker = new Worker (i);
	    workers.add (worker);
	    worker.start ();
	}

	for (Iterator i = startURLS.iterator (); i.hasNext (); ) {
	    URL u = (URL)i.next ();
	    addURL (u);
	}
	
    }

    private void addURL (URL u) {
	synchronized (urlsToGet) {
	    String s = u.toString ();
	    if (history.get (s) != null)
		return;
	    history.put (s, u);
	    urlsToGet.add (u);
	    if (urlsToGet.size () > 0)
		urlsToGet.notify ();
	}
    }

    private URL getURL () {
	synchronized (urlsToGet) {
	    while (urlsToGet.size () == 0) {
		try {
		    urlsToGet.wait ();
		} catch (InterruptedException e) {
		    e.printStackTrace ();
		}
	    }
	    URL u = urlsToGet.removeFirst ();
	    return u;
	}
    }

    protected WebConnection getWebConnection (HTTPHeader header) throws IOException {
	return proxy.getWebConnection (header);
    }

    protected void returnWebConnection (WebConnection wc) {
	proxy.releaseWebConnection (wc);
    }
    
    private class Worker extends Thread {
	private URL currentURL; 
	
	public Worker (int id) {
	    super ("Worker: " + id);
	}
	
	public void run () {
	    while (true) {
		URL u = getURL ();
		System.err.println (this + " got an url: " + u);
		handleURL (u);
		try {
		    Thread.sleep (sleeptime);
		} catch (InterruptedException e) {
		    e.printStackTrace ();
		}
	    }
	}

	private void handleURL (URL u) {
	    this.currentURL = u;

	    HTTPHeader request = new HTTPHeader ();
	    request.setRequestLine ("GET " + u.toString () + " HTTP/1.1");
	    request.setHeader ("Host", u.getHost ());
	    request.setHeader ("User-Agent", spiderId);
	    request.setHeader ("Accept-Encoding", "gzip");

	    WebConnection wc = null;
	    HTTPInputStream contentStream = null;
	    try {
		wc = getWebConnection (request);
		wc.writeHTTPHeader (request);
		contentStream = wc.getInputStream ();
		HTTPHeader response = contentStream.readHTTPHeader ();
		if (wc.chunked ()) {
		    contentStream = new HTTPInputStream (wc.getChunkStream (), proxy);
		}
		handleData (response, wc, contentStream);
	    } catch (IOException e) {
		Integer i = (Integer)failures.get (u);
		int t = i != null ? i.intValue () : 0;
		if (t < 5) {
		    if (contentStream != null) // remove connection
			contentStream.setKeepAlive (false);
		    failures.put (u, new Integer (t + 1));
		    addURL (u); // retry later on
		} else {
		    System.err.println ("unable to connect to: " + u + ": " + e);
		}
	    } finally {
		if (wc != null)
		    returnWebConnection (wc);
	    }
	}

	private void handleData (HTTPHeader response, WebConnection wc, HTTPInputStream contentStream) 
	    throws IOException {
	    String contype = response.getHeader ("Content-Type");
	    long total = 0;
	    byte[] v = new byte[1024];
	    int read;	    
	    long size = wc.dataSize ();
	    
	    if (contype == null || //!contype.equals ("text/html")) {
		!contype.startsWith ("text/html")) {
		if (size < 0) {
		    while ((read = contentStream.read (v)) > 0) {
			total += read;
		    }
		} else {
		    while (total < size && (read = contentStream.read(v)) > 0) {
			total += read;
		    }
		}
	    } else {
		String te = response.getHeader ("Content-Encoding");
		InputStream is = contentStream;
		if (te != null && te.equals ("gzip")) {
		    is = new GZIPInputStream (contentStream);
		} 
		HTMLParser parser = new HTMLParser ();
		HTMLBlock block = null;
		
		if (size < 0) {
		    while ((read = is.read (v)) > 0) {
			total += read;
			handleData (v, parser,block, read);
		    }
		} else {
		    while (total < size && (read = is.read(v)) > 0) {
			total += read;
			handleData (v, parser,block, read);
		    }
		}
		while ((read = is.read (v)) > 0) {
		}
	    }
	}
	
	private void handleData (byte[]v, HTMLParser parser, HTMLBlock block, int read) {
	    byte[] data = v;
	    int length = read;
	    if (block != null && block.restSize () > 0) {
		length += block.restSize ();
		byte[] b = new byte [length];
		block.insertRest (b);
		System.arraycopy (v, 0, b, block.restSize (), read);
		data = b;
	    }
	    parser.setText (data, length);
	    try {
		block = parser.parse ();
		List tokens = block.getTokens ();
		checkTokens (tokens);
	    } catch (HTMLParseException e) {
		System.err.println ("bad HTML: " + e.toString());			
	    }
	}
	
	private void checkTokens (List tokens) {
	    for (int a = 0; a < tokens.size (); a++) {
		Token t = (Token)tokens.get(a);
		if (t.getType () == Token.TAG) {
		    Tag tag = t.getTag ();
		    String nurl = null;
		    if (tag.getLowerCaseType ().equals ("a") || 
			tag.getLowerCaseType ().equals ("area")) {
			nurl = tag.getAttribute ("href");
		    }
		    else if (tag.getLowerCaseType ().equals ("frame") || 
			     tag.getLowerCaseType ().equals ("img")) {
			nurl = tag.getAttribute ("src");
		    }
		    if (nurl != null) 
			doAddURL (nurl);
		}
	    }
	}
	
	private void doAddURL (String nurl) {
	    if (nurl.startsWith ("\"") || nurl.startsWith ("'"))
		nurl = nurl.substring (1);
	    if (nurl.endsWith ("\"") || nurl.endsWith ("'"))
		nurl = nurl.substring (0, nurl.length () - 1);
	    // cut away anchors....
	    int i = nurl.indexOf ("#");
	    if (i > -1)
		nurl = nurl.substring (0, i);
	    URL turl;
	    try {
		turl = new URL (currentURL, nurl);
		String proto = turl.getProtocol ();
		if (proto != null) {		
		    proto = proto.toLowerCase ();
		    if (proto.equals ("http")) {
			addURL (turl);
		    }
		}
	    } catch (MalformedURLException e) {
		//System.err.println ("bad url:" + e);
		// ignore...
	    }
	}
    }
}
